#ifndef NBTV_FORMAT_H
#define NBTV_FORMAT_H

#define NUM_BUILT_IN_FORMATS 3
#define MAX_USR_FORMATS 16

#define NUM_VIDEO_COLORS 9
#define V_COLOR_MONO 0
#define V_COLOR_RED 1
#define V_COLOR_GREEN 2
#define V_COLOR_BLUE 3
#define V_COLOR_CYAN 4
#define V_COLOR_MAGENTA 5
#define V_COLOR_YELLOW 6
#define V_COLOR ORANGE 7
#define V_COLOR_WHITE 8

struct NBTV_channel {
	uint8_t video: 1;
	uint8_t invert_video: 1;
	uint8_t audio: 1;
	uint8_t invert_audio: 1;
	uint8_t sync: 1;
	uint8_t audio_agc:1;
	
	uint8_t invert_channel: 1;
	uint8_t padding: 1;
	
	int8_t channel_gain;
	
	uint8_t video_color;
	uint8_t video_level;
	int8_t video_dc_off;
	int8_t red_gain;
	int8_t green_gain;
	int8_t blue_gain;
	
	uint8_t audio_level;
	int8_t audio_dc_off;
	
	int8_t sync_dc_off;
	uint8_t invert_sync: 1;
	uint8_t line_sync: 1;
	uint8_t pos_line_sync: 1;
	uint8_t frame_sync: 1;
	uint8_t fs_no_ls_f: 1;
	uint8_t fs_no_ls_l: 1;
	uint8_t fs_pulse: 1;
	uint8_t pos_frame_sync: 1;
	double fs_length;
	double ls_length;
	uint8_t fs_level;
	uint8_t ls_level;	
};
	
struct NBTV_line {
	int line_delay;
	int line_length;
	uint8_t line_width: 6;
	uint8_t lock_to_val: 1;
	uint8_t lock_to_line: 1;
	uint8_t lock_val;
	uint8_t lock_line;
	double pc_factor;
};

struct NBTV_format {
	char preset_name[32];
	uint8_t orientation : 1;
	uint8_t left_right_scan: 1;
	uint8_t top_down_scan: 1; 
	uint16_t lines : 13;
	uint16_t pixels_per_line;
	double frame_rate;
	double aspect_ratio;
	uint32_t pixel_clock;
	float line_clock;
	int8_t brightness;
	int8_t contrast;
	double frame_blanking_time;
	int8_t frame_blanking_value;
	double line_blanking_time;
	int8_t line_blanking_value;
	uint16_t first_usable_line;
	uint16_t last_usable_line;
	uint16_t imgW,imgH;
	struct NBTV_line *le_lines;
	struct NBTV_channel channels[4];
	uint8_t fields_per_frame: 6;
	uint8_t interlace: 1;
	uint8_t fsc: 1;
	union {
		uint8_t fsc_colors[4];
		uint8_t line_interlace_order[4];
	} mfpf;
};

void load_formats();
void set_NBTV_format(int index);

extern struct NBTV_format built_in_formats[NUM_BUILT_IN_FORMATS];
extern struct NBTV_format *nbtv_fmt;
extern struct NBTV_format user_formats[MAX_USR_FORMATS];
extern int format_index;
extern int last_user_fmt;

void add_format(char *name);
void save_formats();
void delete_format();

void clrfi();

void nbtv_format_set_orientation(char o);
void nbtv_format_set_left_right_scan(char d);
void nbtv_format_set_top_down_scan(char d);
void nbtv_format_set_lines(int lc);
void nbtv_format_set_ppl(int ppl);
void nbtv_format_set_framerate(double fps);
void nbtv_format_set_aspect(double aspect);
void nbtv_format_set_pixel_clock(unsigned int pc);
void nbtv_format_set_line_clock(double lc);
void nbtv_format_set_brightness(int8_t br);
void nbtv_format_set_contrast(int8_t cr);
void nbtv_format_set_frame_blank_time(double bt);
void nbtv_format_set_frame_blank_value(int8_t fb);
void nbtv_format_set_line_blank_time(double bt);
void nbtv_format_set_line_blank_value(int8_t lb);
void nbtv_format_set_usable_lines(int lower, int higher);
void nbtv_format_set_screen_aspect(double aspect);

#endif