/* 	Copyright John D. "smeezekitty" 2017
	
	FreeNBTV is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FreeNBTV is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FreeNBTV.  If not, see <http://www.gnu.org/licenses/>.
*/

/* This file mostly handles changing the NBTV parameters ahd presets */

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include "nbtv_format.h"
#include "input.h"
#include "nbtv_gen.h"
#include "gui.h"

/*Calculate the pixel clock for a particular format setting*/
#define CALC_PC() (double)nbtv_fmt->pixels_per_line * 1000000.0/((1000000.0/nbtv_fmt->line_clock) - (double)nbtv_fmt->line_blanking_time)

struct NBTV_format built_in_formats[NUM_BUILT_IN_FORMATS];
struct NBTV_format user_formats[MAX_USR_FORMATS];
struct NBTV_format n;
struct NBTV_format *nbtv_fmt;	
int format_index;
int last_user_fmt = 0;

/*Calculate the pixel clock for a particular format setting*/
double calc_pc(struct NBTV_format *fmt){
	return (double)fmt->pixels_per_line * 1000000.0/((1000000.0/fmt->line_clock) - (double)fmt->line_blanking_time);
}

void set_NBTV_format(int index){
	if(index > NUM_BUILT_IN_FORMATS + last_user_fmt) return;
	format_index = index;
	if(!index) return; //format 0 is the "Custom" entry. It does not have an associated entry in the format table
	if(--index >= NUM_BUILT_IN_FORMATS){
		memcpy(nbtv_fmt, &user_formats[index - NUM_BUILT_IN_FORMATS], sizeof(struct NBTV_format));
		return;
	}
	memcpy(nbtv_fmt, &built_in_formats[index], sizeof(struct NBTV_format));
}
//"CLeaR Format Index" -- sets the entry to "Custom" if any parameter is changed by the user
void clrfi(){
	format_index = 0;
	gui_update_format_selector();
}
void select_NBTV_format(GtkWidget *cb, gpointer bla){}

void init_built_in_formats(){ 
	struct NBTV_format *bif = built_in_formats;
	//memset(built_in_formats, 0, sizeof(built_in_formats));
	strcpy(bif->preset_name, "NBTVA");
	bif->orientation = 1;
	bif->lines = 32;
	bif->pixels_per_line = 48;
	bif->frame_rate = 12.5;
	bif->aspect_ratio = 0.67;
	bif->line_clock = 400;
	bif->frame_blanking_time = 0;
	bif->line_blanking_time = 175.0;
	bif->first_usable_line = 0;
	bif->last_usable_line = 31;
	bif->imgW = 32;
	bif->imgH = 48;
	bif->brightness = 0;
	bif->contrast = 0;
	bif->channels->video = 1;
	bif->channels->sync = 1;
	bif->channels->video_color = V_COLOR_MONO;
	bif->channels->video_level = 80;
	bif->channels->line_sync = 1;
	bif->channels->frame_sync = 1;
	bif->channels->fs_no_ls_f = 1;
	bif->channels->fs_length = 0;
	bif->channels->ls_length = 175.0;
	bif->channels->ls_level = 60.0;
	bif->pixel_clock = calc_pc(bif);
	bif++;
	strcpy(bif->preset_name, "30 Line Barid");
	bif->orientation = 1;
	bif->lines = 30;
	bif->pixels_per_line = 45;
	bif->frame_rate = 12.5;
	bif->aspect_ratio = 0.67;
	bif->line_clock = 375;
	bif->frame_blanking_time = 0;
	bif->line_blanking_time = 175.0;
	bif->first_usable_line = 0;
	bif->last_usable_line = 29;
	bif->imgW = 30;
	bif->imgH = 45;
	bif->brightness = 0;
	bif->contrast = 0;
	bif->channels->video = 1;
	bif->channels->sync = 0;
	bif->channels->video_color = V_COLOR_MONO;
	bif->channels->video_level = 80;
	bif->channels->line_sync = 0;
	bif->channels->frame_sync = 0;
	bif->channels->fs_no_ls_f = 0;
	bif->channels->fs_length = 0;
	bif->channels->ls_length = 0.0;
	bif->channels->ls_level = 0;
	bif->pixel_clock = calc_pc(bif);
	bif++;
	strcpy(bif->preset_name, "60 Line Portrait");
	bif->orientation = 0;
	bif->lines = 60;
	bif->pixels_per_line = 80;
	bif->frame_rate = 12.5;
	bif->aspect_ratio = 1.33;
	bif->line_clock = 750;
	bif->frame_blanking_time = 0;
	bif->line_blanking_time = 175.0;
	bif->first_usable_line = 0;
	bif->last_usable_line = 79;
	bif->imgW = 80;
	bif->imgH = 60;
	bif->brightness = 0;
	bif->contrast = 0;
	bif->channels->video = 1;
	bif->channels->sync = 1;
	bif->channels->video_color = V_COLOR_MONO;
	bif->channels->video_level = 80;
	bif->channels->line_sync = 1;
	bif->channels->frame_sync = 1;
	bif->channels->fs_no_ls_f = 1;
	bif->channels->fs_length = 0;
	bif->channels->ls_length = 175.0;
	bif->channels->ls_level = 60.0;
	bif->pixel_clock = calc_pc(bif);
}
#define SM(x) (strcmpi(key, (x))==0)
/*Loads channel specific stuff -- called by load_formats()*/
void load_channel_spec(int channel, struct NBTV_format *f, FILE *fi, int *line){
	char buffer[64];
	char key[32],p1[16],p2[16],p3[16];
	struct NBTV_channel *ch = &f->channels[channel];
	int v;
	uint8_t v8;
	while(fgets(buffer, 64, fi)){
		*key = *p1 = *p2 = *p3 = 0;
		sscanf(buffer, "%s %s %s %s", key, p1, p2, p3);
		(*line)++;
		debug("[Ch %d] Got %s (%s, %s, %s) <%d>\n", channel, key, p1, p2, p3, *line);
		if(SM("enable")){
			sscanf(p1, "0x%X", &v);
			v8 = v;
			ch->video = !!(v8 & 0x40);
			ch->invert_video = !!(v8 & 0x20);
			ch->audio = !!(v8 & 0x10);
			ch->invert_audio = !!(v8 & 0x08);
			ch->sync = !!(v8 & 0x04);
			ch->audio_agc = !!(v8 & 0x02);
			ch->invert_channel = v8&1;
			continue;
		}
		if(SM("gain")){
			ch->channel_gain = atoi(p1);
			continue;
		}
		if(SM("color")){
			ch->video_color = atoi(p1);
			continue;
		}
		//if(SM("color-gain
		if(SM("vlevel")){
			ch->video_level = atoi(p1);
			continue;
		}
		if(SM("vsco")){
			ch->video_dc_off = atoi(p1);
			continue;
		}
		if(SM("alevel")){
			ch->audio_level = atoi(p1);
			continue;
		}
		if(SM("adco")){
			ch->audio_dc_off = atoi(p1);
			continue;
		}
		if(SM("sdco")){
			ch->sync_dc_off = atoi(p1);
			continue;
		}
		if(SM("sync-flags")){
			sscanf(p1, "0x%X", &v);
			v8 = v;
			ch->invert_sync = !!(v8 & 0x80);
			ch->line_sync = !!(v8 & 0x40);
			ch->pos_line_sync = !!(v8 & 0x20);
			ch->frame_sync = !!(v8 & 0x10);
			ch->fs_no_ls_f = !!(v8 & 0x08);
			ch->fs_no_ls_l = !!(v8 & 0x04);
			ch->fs_pulse = !!(v8 & 0x02);
			ch->pos_frame_sync = v8&1;
			continue;
		}
		if(SM("fs-length")){
			sscanf(p1, "%f", &ch->fs_length);
			continue;
		}
		if(SM("ls-length")){
			sscanf(p1, "%f", &ch->ls_length);
			continue;
		}
		if(SM("fs-level")){
			ch->fs_level = atoi(p1);;
			continue;
		}
		if(SM("ls-level")){
			ch->ls_level = atoi(p1);
			continue;
		}
		if(SM("color-gain")){
			ch->red_gain = atoi(p1);
			ch->green_gain = atoi(p2);
			ch->blue_gain = atoi(p3);
			continue;
		}
		if(SM("<END") && !strcmpi(p1, "CHANNEL>")){
			return;
		}
		if(SM("<END>")){
			error("Unexpected <END> of entry line %d\n", *line);
			return;
		}
		error("Unknown token %s line %d\n", key, *line);
	}
	error("Expected <END CHANNEL> token line %d\n", *line);
}
/*Loads user NBTV format presets from nbtvgen.fmt*/			
void load_formats(){
	char buffer[64];
	char key[32];
	char p1[8],p2[8],p3[8];
	int v1,v2;
	int line = 0;
	init_built_in_formats();
	nbtv_fmt = &n;
	struct NBTV_format *f;
	set_NBTV_format(1);
	FILE *fi = fopen("nbtvgen.fmt", "r");
	if(!fi){
		warn("Failed to open nbtvgen.fmt for reading\n");
		return;
	}
	fgets(buffer, 64, fi);
	if(sscanf(buffer, "V%d.%d", &v1, &v2) != 2){
		warn("Expected version string on line 1");
		goto skv;
	} else line = 1;
	if(v1 > 1 || v2 > 0){
		warn("Warning: Possible version mismatch\n");
	}
	while(fgets(buffer, 64, fi)){
		skv:
		*key = *p1 = *p2 = *p3 = 0;
		sscanf(buffer, "%s %s %s %s\n", key, p1, p2, p3);
		line++;
		debug("Got %s (%s, %s, %s) <%d>\n", key, p1, p2, p3, line);
		if(SM("<START>")){
			f = &user_formats[last_user_fmt++];
			debug("Last user fmt now = %d\n", last_user_fmt);
			strcpy(f->preset_name, p1+1);
			v1 = (int)strchr(f->preset_name, '\"');
			if(v1){
				v1 -= (int)f->preset_name;
				f->preset_name[v1] = 0;
			}
			continue;
		}
		if(SM("scan")){
			v1 = atoi(p1);
			f->orientation = !!(v1 & 4);
			f->left_right_scan = !!(v1 & 2);
			f->top_down_scan = v1 & 1;
			continue;
		}
		if(SM("lines")){
			f->lines = atoi(p1);
			continue;
		}
		if(SM("pixels-per-line")){
			f->pixels_per_line = atoi(p1);
			continue;
		}
		if(SM("framerate")){
			f->frame_rate = atof(p1);
			continue;
		}
		if(SM("aspect-ratio")){
			f->frame_rate = atof(p1);
			continue;
		}
		if(SM("pixel-clock")){
			f->pixel_clock = atoi(p1);
			continue;
		}
		if(SM("line-clock")){
			f->line_clock = atof(p1);
			continue;
		}
		if(SM("brightness")){
			f->brightness = atoi(p1);
			continue;
		}
		if(SM("contrast")){
			f->contrast = atoi(p1);
			continue;
		}
		if(SM("fbt")){
			f->frame_blanking_time = atof(p1);
			continue;
		}
		if(SM("fbv")){
			f->frame_blanking_value = atoi(p1);
			continue;
		}
		if(SM("lbt")){
			f->line_blanking_time = atof(p1);
			continue;
		}
		if(SM("lbv")){
			f->line_blanking_value = atoi(p1);
			continue;
		}
		if(SM("usable-lines")){
			f->first_usable_line = atoi(p1);
			f->last_usable_line = atoi(p2);
			continue;
		}
		if(SM("input-img-dem")){
			sscanf(p1, "%dX%d", &f->imgW, &f->imgH);
			continue;
		}
		if(SM("<CHANNEL>")){
			load_channel_spec(atoi(p1), f, fi, &line);
			continue;
		}
		if(SM("<END>")){
			break;
		}
		error("Unknown token %s line %d\n", key, line);
	}
	fclose(fi);
}
/*Save user formats to nbtvgen.fmt*/
void save_formats(){
	FILE *fo = fopen("nbtvgen.fmt", "w");
	struct NBTV_format *f;
	if(!fo){
		error("Failed to open nbtvgen.fmt for writing\n");
		return;
	}
	int i,j;
	fprintf(fo, "V1.0\n");
	for(i = 0;i < last_user_fmt;i++){
		f = &user_formats[i];
		fprintf(fo,"<START> \"%s\"\n", f->preset_name);
		fprintf(fo,"scan %d\n", f->orientation << 2 | f->left_right_scan << 1 | f->top_down_scan);
		fprintf(fo,"lines %d\n", f->lines);
		fprintf(fo,"pixels-per-line %d\n", f->pixels_per_line);
		fprintf(fo,"framerate %0.2f\n", f->frame_rate);
		fprintf(fo,"aspect-ratio %0.2f\n", f->aspect_ratio);
		fprintf(fo,"pixel-clock %d\n", f->pixel_clock);
		fprintf(fo,"line-clock %0.1f\n", f->line_clock);
		fprintf(fo,"brightness %d\n", f->brightness);
		fprintf(fo,"contrast %d\n", f->contrast);
		fprintf(fo,"fbt %0.1f\n", f->frame_blanking_time);
		fprintf(fo,"fbv %d\n", f->frame_blanking_value);
		fprintf(fo,"lbt %0.1f\n", f->line_blanking_time);
		fprintf(fo,"lbv %d\n", f->line_blanking_value);
		fprintf(fo,"usable-lines %d %d\n", f->first_usable_line, f->last_usable_line);
		fprintf(fo,"input-img-dem %dX%d\n", f->imgW, f->imgH);
		for(j = 0;j < 4;j++){
			struct NBTV_channel *ch = &f->channels[j];
			if(!(ch->video || ch->audio || ch->sync)){
				continue;
			}
			fprintf(fo,"<CHANNEL> %d\n", j);
			fprintf(fo,"enable 0x%X\n", ch->video << 6 | ch->invert_video << 5 | ch->audio << 4 | ch->invert_audio << 3 | ch->sync << 2 | ch->audio_agc << 1 | ch->invert_channel);
			fprintf(fo,"gain %d\n", ch->channel_gain);
			if(ch->video){
				if(ch->video_color) fprintf(fo,"color %d\n", ch->video_color);
				if(ch->video_level) fprintf(fo,"vlevel %d\n", ch->video_level);
				if(ch->video_dc_off) fprintf(fo,"vdco %d\n", ch->video_dc_off);
				fprintf(fo,"color-gain %d %d %d\n", ch->red_gain, ch->green_gain, ch->blue_gain);
			}
			if(ch->audio){
				if(ch->audio_level) fprintf(fo,"alevel %d\n", ch->audio_level);
				if(ch->audio_dc_off) fprintf(fo,"adco %d\n", ch->audio_dc_off);
			}
			if(ch->sync){
				if(ch->sync_dc_off) fprintf(fo,"sdco %d\n", ch->sync_dc_off);
				fprintf(fo,"sync-flags 0x%X\n", ch->invert_sync << 7 | ch->line_sync << 6 | ch->pos_line_sync << 5 | ch->frame_sync << 4 | ch->fs_no_ls_f << 3 | ch->fs_no_ls_l << 2 | ch->fs_pulse << 1 | ch->pos_frame_sync);
				fprintf(fo,"fs-length %0.1f\n", ch->fs_length);
				fprintf(fo,"ls-length %0.1f\n", ch->ls_length);
				if(ch->fs_level) fprintf(fo,"fs-level %d\n", ch->fs_level);
				if(ch->ls_level) fprintf(fo,"ls-level %d\n", ch->ls_level);
			}
			fprintf(fo, "<END CHANNEL> %d\n", j);
		}	
		if(f->fields_per_frame > 1){
			fprintf(fo,"fields-per-frame %d %s\n", f->fields_per_frame, f->fsc? "FSC": "INTERLACE");
			if(f->interlace){
				fprintf(fo,"order %d", f->mfpf.line_interlace_order[0]);
				for(j = 1;j < f->fields_per_frame;j++){
					fprintf(fo," %d", f->mfpf.line_interlace_order[j]);
				}
			} else if(f->fsc){
				fprintf(fo,"colors %d", f->mfpf.line_interlace_order[0]);
				for(j = 1;j < f->fields_per_frame;j++){
					fprintf(fo," %d", f->mfpf.line_interlace_order[j]);
				}
			}
			fprintf(fo,"\n");
		}
		if(f->le_lines){
			fprintf(fo,"<LINE EDITOR BLOCK>\n");
			for(j = 0;j < f->lines;j++){
				fprintf(fo,"%d %d %d %d %X %0.2f", j, f->le_lines[j].line_delay, f->le_lines[j].line_width, f->le_lines[j].lock_to_val << 1 | f->le_lines[j].lock_to_line, f->le_lines[j].pc_factor);
				if(f->le_lines[j].lock_to_val || f->le_lines[j].lock_to_line){
					fprintf(fo,"%d %d\n", f->le_lines[j].lock_val, f->le_lines[j].lock_line);
				} else fprintf(fo,"\n");
			}
			fprintf(fo,"<LINE EDITOR BLOCK END>\n");
		}
		fprintf(fo,"<END>\n");
	}
	fclose(fo);
}
/*Adds a new preset format called "name" with the current parameters to the list*/
void add_format(char *name){
	int i;
	for(i = 0;i < NUM_BUILT_IN_FORMATS;i++){
		if(!strcmpi(name, built_in_formats[i].preset_name)){
			error("Error: Name %s already exists\n", name);
			return;
		}
	}
	for(i = 0;i < last_user_fmt;i++){
		if(!strcmpi(name, user_formats[i].preset_name)){
			error("Error: Name %s already exists\n", name);
			return;
		}
	}	
	if(last_user_fmt >= MAX_USR_FORMATS) return;
	struct NBTV_format *target = &user_formats[last_user_fmt++];
	*target = *nbtv_fmt;
	if(nbtv_fmt->le_lines){
		target->le_lines = malloc(target->lines * sizeof(struct NBTV_line));
		memcpy(target->le_lines, nbtv_fmt->le_lines, target->lines * sizeof(struct NBTV_line));
	}
	strcpy(target->preset_name, name);
	set_NBTV_format(last_user_fmt + NUM_BUILT_IN_FORMATS - 1);
	gui_update_format(nbtv_fmt);
}
/*Delete the current preset format from the list (if it isn't built in) and sets the dropdown to "Custom"*/
void delete_format(){
	if(format_index < NUM_BUILT_IN_FORMATS + 1){
		error("Format is a built in preset");
	}
	if(format_index-1 != NUM_BUILT_IN_FORMATS + last_user_fmt){ /*If it's the last one on the list, we don't need to split the array*/
		/*Otherwise splice it by shifting all elements ahead of the deleted element back one - this keeps it contiguous*/
		memmove(&user_formats[format_index - NUM_BUILT_IN_FORMATS - 1], &user_formats[format_index - NUM_BUILT_IN_FORMATS], sizeof(*user_formats) * ((last_user_fmt - format_index) + 1)); 
	}
	last_user_fmt--;
	set_NBTV_format(0);
	gui_update_format(nbtv_fmt);
}
/*Update Width&Height -- sets parameters used by the input layer to scale the image*/
void update_WH(){
	if(nbtv_fmt->orientation){ /*Portrait*/
		nbtv_fmt->imgH = nbtv_fmt->pixels_per_line;
		nbtv_fmt->imgW = nbtv_fmt->last_usable_line - nbtv_fmt->first_usable_line + 1;
	} else { /*Landscape*/
		nbtv_fmt->imgW = nbtv_fmt->pixels_per_line;
		nbtv_fmt->imgH = nbtv_fmt->last_usable_line - nbtv_fmt->first_usable_line + 1;
	}
	format_changed = 1;
}
void nbtv_format_set_orientation(char o){
	clrfi();
	if(nbtv_fmt->orientation == !!o) return;
	nbtv_fmt->orientation = o;
	update_WH();
	nbtv_fmt->aspect_ratio = 1/nbtv_fmt->aspect_ratio;
	gui_update_format(nbtv_fmt);
	format_changed = 1;
}
void nbtv_format_set_left_right_scan(char d){
	clrfi();
	nbtv_fmt->left_right_scan = d;
	gui_update_format(nbtv_fmt);
	format_changed = 1;
}
void nbtv_format_set_top_down_scan(char d){
	clrfi();
	nbtv_fmt->top_down_scan = d;
	gui_update_format(nbtv_fmt);
	format_changed = 1;
}

void nbtv_format_set_lines(int lc){
	clrfi();
	if(lc < 1 || lc > 240)return;
	nbtv_fmt->lines = lc;
	nbtv_fmt->line_clock = nbtv_fmt->frame_rate * (double)nbtv_fmt->lines;
	nbtv_fmt->last_usable_line = lc-1;
	nbtv_fmt->pixel_clock = CALC_PC();
	update_WH();
	gui_update_format(nbtv_fmt);
	format_changed = 1;
}
void nbtv_format_set_ppl(int ppl){
	clrfi();
	if(ppl < 1 || ppl > 720)return;
	nbtv_fmt->pixels_per_line = ppl;
	nbtv_fmt->pixel_clock = CALC_PC();
	update_WH();
	gui_update_format(nbtv_fmt);
	format_changed = 1;
	
}
void nbtv_format_set_framerate(double fps){
	clrfi();
	if(fps < 0.1 || fps > 120) return;
	nbtv_fmt->frame_rate = fps;
	nbtv_fmt->line_clock = nbtv_fmt->frame_rate * (double)nbtv_fmt->lines;
	nbtv_fmt->pixel_clock = CALC_PC();
	gui_update_format(nbtv_fmt);
	format_changed = 1;
}
void nbtv_format_set_aspect(double aspect){
	clrfi();
	nbtv_fmt->aspect_ratio = aspect;
	gui_update_format(nbtv_fmt);
	format_changed = 1;
}
void nbtv_format_set_pixel_clock(unsigned int pc){
	clrfi();
	if(pc > 256000)return;
	nbtv_fmt->pixels_per_line = round(pc * ((1000000/(double)nbtv_fmt->line_clock) - (double)nbtv_fmt->line_blanking_time) / 1000000.0);
	nbtv_fmt->pixel_clock = CALC_PC();
	update_WH();
	gui_update_format(nbtv_fmt);
	format_changed = 1;
}
void nbtv_format_set_line_clock(double lc){
	clrfi();
	if(lc > 64000)return;
	nbtv_fmt->line_clock = lc;
	nbtv_fmt->frame_rate = (double)lc/(double)nbtv_fmt->lines;
	nbtv_fmt->pixel_clock = CALC_PC();
	gui_update_format(nbtv_fmt);
	format_changed = 1;
}
void nbtv_format_set_brightness(int8_t br){
	clrfi();
	nbtv_fmt->brightness = br;
	gui_update_format(nbtv_fmt);
	format_changed = 1;
}
void nbtv_format_set_contrast(int8_t cr){
	clrfi();
	nbtv_fmt->contrast = cr;
	gui_update_format(nbtv_fmt);
	format_changed = 1;
}
void nbtv_format_set_frame_blank_time(double bt){
	clrfi();
	nbtv_fmt->frame_blanking_time = bt;
	gui_update_format(nbtv_fmt);
	format_changed = 1;
}
void nbtv_format_set_frame_blank_value(int8_t fb){
	clrfi();
	nbtv_fmt->frame_blanking_value = fb;
	gui_update_format(nbtv_fmt);
	format_changed = 1;
}
void nbtv_format_set_line_blank_time(double bt){
	clrfi();
	nbtv_fmt->line_blanking_time = bt;
	nbtv_fmt->pixel_clock = CALC_PC();
	gui_update_format(nbtv_fmt);
	format_changed = 1;
}
void nbtv_format_set_line_blank_value(int8_t lb){
	clrfi();
	nbtv_fmt->line_blanking_value = lb;
	gui_update_format(nbtv_fmt);
	format_changed = 1;
}
void nbtv_format_set_usable_lines(int lower, int higher){
	clrfi();
	if(higher < 1){higher = 1;}
	if(lower >= higher){lower = higher - 1;}
	if(lower < 0){lower = 0;}
	if(higher > nbtv_fmt->lines-1){
		higher=nbtv_fmt->lines - 1;
	}
	nbtv_fmt->first_usable_line = lower;
	nbtv_fmt->last_usable_line = higher;
	update_WH();
	format_changed = 1;
}
void nbtv_format_set_screen_aspect(double aspect){
	clrfi();
	nbtv_fmt->aspect_ratio = aspect;
	format_changed = 1;
}